"use strict";
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
var express_1 = __importDefault(require("express"));
var cors_1 = __importDefault(require("cors"));
var moment_1 = __importDefault(require("moment"));
var log_1 = __importDefault(require("./log"));
var routes_1 = __importDefault(require("./routes"));
var path_to_regexp_1 = require("path-to-regexp");
var path_1 = __importDefault(require("path"));
var Server = (function () {
    function Server(port, host, data) {
        this.port = port;
        this.host = host;
        this.app = express_1["default"]();
        this.routes = new routes_1["default"](data);
        this.bindMiddleware();
        this.bindDashboardRoute();
        this.bindAppRoute();
    }
    Server.prototype.bindMiddleware = function () {
        this.app.set("views", path_1["default"].resolve(__dirname, "./views"));
        this.app.set("view engine", "ejs");
        this.app.use(cors_1["default"]());
        this.app.use(function (req, res, next) {
            log_1["default"].log(moment_1["default"].utc().format() + " : " + log_1["default"].blue("[" + req.method + "] " + req.url));
            next();
        });
    };
    Server.prototype.bindDashboardRoute = function () {
        var _this = this;
        this.app.get("/", function (req, res) {
            return res.render("index", {
                port: _this.port,
                host: _this.host,
                routes: _this.routes.routes
            });
        });
    };
    Server.prototype.bindAppRoute = function () {
        var _this = this;
        this.app.all("*", function (req, res) {
            var _a = __read(Object.entries(_this.routes.routes).find(function (_a) {
                var _b = __read(_a, 1), path = _b[0];
                return _this.doesMatchPath(path, req.url);
            }) || [null, null], 2), path = _a[0], methodList = _a[1];
            if (path === null) {
                return res.status(404).json(null);
            }
            if (!methodList[req.method.toLowerCase()]) {
                log_1["default"].log(log_1["default"].red("route " + path + " does not have method " + req.method));
                return res.status(404).json(null);
            }
            return res.jsonp(methodList[req.method.toLowerCase()]);
        });
        return this;
    };
    Server.prototype.start = function () {
        var _this = this;
        this.server = this.app.listen(this.port, this.host, function () {
            log_1["default"].log("App Listening on " + log_1["default"].green("http://" + _this.host + ":" + _this.port));
        });
        return this;
    };
    Server.prototype.stop = function () {
        var _a;
        (_a = this.server) === null || _a === void 0 ? void 0 : _a.close();
    };
    Server.prototype.reload = function () {
        log_1["default"].log(log_1["default"].blue(".reload() has been deprecated."));
    };
    Server.prototype.doesMatchPath = function (rawPath, serverPath) {
        return !!path_to_regexp_1.match(rawPath, { decode: decodeURIComponent })(serverPath);
    };
    Server.create = function (port, host, data) {
        return new Server(port, host, data);
    };
    return Server;
}());
exports["default"] = Server;
//# sourceMappingURL=server.js.map